@@ -36,13 +36,6 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: inet_pton.c,v 1.3 2007/09/27 18:02:20 gianluca Exp $ */
-
-#ifndef lint
-static const char rcsid[] _U_ =
-     "@(#) $Header: /tcpdump/master/libpcap/Win32/Src/inet_pton.c,v 1.3 2007/09/27 18:02:20 gianluca Exp $";
-#endif
-
 #include <errno.h>
 
 #ifdef WIN32
